from flask import Blueprint
from flask import request
from flask import abort
from flask import Response
from . import baseAPI
from . import deviceController
from . import gpioservice


deviceApiBlueprint = Blueprint('deviceApi', __name__)

@deviceApiBlueprint.route('/trainmote/api/v1/device/restart', methods=["POST"])
def restartDevice():
    gpioservice.clean()
    timer = deviceController.RestartTimer()
    timer.start()
    return "", 200

@deviceApiBlueprint.route('/trainmote/api/v1/device/shutdown', methods=["POST"])
def shutdownDevice():
    gpioservice.clean()
    timer = deviceController.ShutDownTimer()
    timer.start()
    return "", 200

@deviceApiBlueprint.route('/trainmote/api/v1/device/update', methods=["POST"])
def updateDevice():
    gpioservice.clean()
    deviceController.update()
    return "", 200



from typing import Optional

class ConfigModel():

    def __init__(
        self,
        uid: int,
        switchPowerRelais: Optional[int],
        powerRelais: Optional[int],
        stateRelais: Optional[int]
    ):
        self.uid = uid
        self.switchPowerRelais = switchPowerRelais
        self.powerRelais = powerRelais
        self.stateRelais = stateRelais

    def to_dict(self):
        return {
            "uid": self.uid,
            "switchPowerRelais": self.switchPowerRelais,
            "powerRelais": self.powerRelais,
            "stateRelais": self.stateRelais
        }

    def containsPin(self, pin: int) -> bool:
        isSwitchPowerRelais = self.switchPowerRelais is not None and self.switchPowerRelais == pin
        isPowerRelais = self.powerRelais is not None and self.powerRelais == pin
        isStateRelais = self.stateRelais is not None and self.stateRelais == pin
        return isSwitchPowerRelais or isPowerRelais or isStateRelais



from subprocess import call
import time
from threading import Thread
import os
import sys

def shutdown():
    call("sudo nohup shutdown -h now", shell=True)

class ShutDownTimer(Thread):
    def run(self):
        time.sleep(2)
        shutdown()

def restart():
    call("sudo reboot", shell=True)

class RestartTimer(Thread):
    def run(self):
        time.sleep(2)
        restart()

def __performUpdate():
    call("pip3 install trainmote-module-felix-nievelstein-de -U", shell=True)
    os.execv(sys.argv[0], [sys.argv[0]])

def update():
    timer = UpdateTimer()
    timer.start()

class UpdateTimer(Thread):
    def run(self):
        time.sleep(2)
        __performUpdate()



from flask import Blueprint
from flask import request
from flask import abort
from flask import Response
from . import baseAPI
from . import gpioservice
from .validator import Validator
import json
from .databaseControllerModule import DatabaseController


stopPointApi = Blueprint('stopPointApi', __name__)
##
# Endpoint StopPoint
##

@stopPointApi.route('/trainmote/api/v1/stoppoint/<stop_id>', methods=["PATCH"])
def setStop(stop_id: str):
    if stop_id is None:
        abort(400)
    try:
        return gpioservice.setStop(stop_id), 200, baseAPI.defaultHeader()
    except ValueError as e:
        return json.dumps({"error": str(e)}), 400, baseAPI.defaultHeader()


@stopPointApi.route('/trainmote/api/v1/stoppoint/<stop_id>', methods=["DELETE"])
def deleteStop(stop_id: str):
    if stop_id is None:
        abort(400)
    DatabaseController().deleteStopModel(int(stop_id)), 205, baseAPI.defaultHeader()
    return 'ok'


@stopPointApi.route('/trainmote/api/v1/stoppoint', methods=["POST"])
def addStop():
    mJson = request.get_json()
    if mJson is not None:
        if Validator().validateDict(mJson, "stop_scheme") is False:
            abort(400)

        config = DatabaseController().getConfig()
        if config is not None and config.containsPin(mJson["id"]):
            return json.dumps({"error": "Pin is already in use as power relais"}), 409, baseAPI.defaultHeader()

        try:
            return gpioservice.createStop(mJson), 201, baseAPI.defaultHeader()
        except ValueError as e:
            return json.dumps({"error": str(e)}), 400, baseAPI.defaultHeader()
    else:
        abort(400)


@stopPointApi.route('/trainmote/api/v1/stoppoint/all', methods=["GET"])
def getAllStops():
    return Response(gpioservice.getAllStopPoints(), mimetype="application/json"), 200, baseAPI.defaultHeader()


@stopPointApi.route('/trainmote/api/v1/stoppoint/<stop_id>', methods=["GET"])
def stop(stop_id: str):
    if stop_id is None:
        abort(400)
    return gpioservice.getStop(stop_id), 200, baseAPI.defaultHeader()
